
#include "lj.h"
#include "datatypes.h"

#include "unes.h"

extern uint32   ppuScanline;
extern uint32 scalePal[256];
extern VirtualNES Vnes;

#include <tapwave.h>

// Assertion Support
static void
assert_check(int error, const char* message)
{
    if (error) {
        ErrDisplayFileLineMsg("PalmOS", (UInt16) error, message);
    }
}

#define xassert(error) {assert_check(error, #error);}

static void showError(const char* aWhat, Err aError) {
    ErrDisplayFileLineMsg("oink", (UInt16) aError, aWhat);
}

extern TwGfxType *lj_mGfxLib;

extern TwGfxSurfaceType *lj_curRenderingScreen,*lj_RenderingScreen[2];
extern uint16 *lj_curRenderingScreenPtr,*lj_RenderingScreenPtr[2];
extern uint32 lj_curRenderingScreenIndex;


extern uint32 lj_renderingmode;

void XlatNESBufferPAL (unsigned char *src, unsigned char *dest, uint8 *nesppupal)
{	
TwGfxPointType destLocation = {0, 0};
    TwGfxSurfaceType* displaySurface;
	TwGfxRectType mDisp;
	mDisp.x=0;
	mDisp.y=0;
	mDisp.w=256;
	mDisp.h=240;
    // Get palmos display surface:
    xassert(TwGfxGetPalmDisplaySurface(lj_mGfxLib, &displaySurface));	
    
	switch (lj_renderingmode)
	{
		case 0:
		{						
			destLocation.x=112;
			destLocation.y=40;
			if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))			
					xassert(TwGfxBitBlt(displaySurface, &destLocation, lj_curRenderingScreen, &mDisp));
					
				
			break;
			}
		case 1:
		{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-342)>>1;
				mDispDest.y=0;
				mDispDest.w=342;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}
		case 2:
		{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-342)>>1;
				mDispDest.y=0;
				mDispDest.w=342;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}		
		case 3:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}	
		case 4:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}							
									
		case 5:
			{
				TwGfxRectType mDispDest;
				mDisp.x=(256-240)>>1;
				mDisp.y=(240-160)>>1;
				mDisp.w=240;
				mDisp.h=160;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}							
		case 6:
			{
				TwGfxRectType mDispDest;
				mDisp.x=(256-240)>>1;
				mDisp.y=(240-160)>>1;
				mDisp.w=240;
				mDisp.h=160;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}							
			
	}
	lj_curRenderingScreenIndex^=1;
	lj_curRenderingScreen=lj_RenderingScreen[lj_curRenderingScreenIndex];
	lj_curRenderingScreenPtr=lj_RenderingScreenPtr[lj_curRenderingScreenIndex];
}
void XlatNESBufferNTSC (unsigned char *src, unsigned char *dest, uint8 *nesppupal)
{
	TwGfxPointType destLocation = {0, 0};
    TwGfxSurfaceType* displaySurface;
	TwGfxRectType mDisp;
	mDisp.x=0;
	mDisp.y=8;
	mDisp.w=256;
	mDisp.h=224;
    // Get palmos display surface:
    xassert(TwGfxGetPalmDisplaySurface(lj_mGfxLib, &displaySurface));	
    
	switch (lj_renderingmode)
	{
		case 0:
		{						
			destLocation.x=112;
			destLocation.y=48;
			if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
				xassert(TwGfxBitBlt(displaySurface, &destLocation, lj_curRenderingScreen, &mDisp));				
			break;			
		}
		case 1:
		{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-366)>>1;
				mDispDest.y=0;
				mDispDest.w=366;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}
		case 2:
		{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-366)>>1;
				mDispDest.y=0;
				mDispDest.w=366;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}
			
	
		case 3:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}							
		case 4:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}							
			
		case 5:
			{
				TwGfxRectType mDispDest;
				mDisp.x=(256-240)>>1;
				mDisp.y=(224-160)>>1;
				mDisp.w=240;
				mDisp.h=160;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}	
		case 6:
			{
				TwGfxRectType mDispDest;
				mDisp.x=(256-240)>>1;
				mDisp.y=(224-160)>>1;
				mDisp.w=240;
				mDisp.h=160;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(lj_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, lj_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}				

	}
	lj_curRenderingScreenIndex^=1;
	lj_curRenderingScreen=lj_RenderingScreen[lj_curRenderingScreenIndex];
	lj_curRenderingScreenPtr=lj_RenderingScreenPtr[lj_curRenderingScreenIndex];
}
void XlatNESBufferLine (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 scanl)
{
/*	uint8 *source = (uint8*)(src);//Vnes.var.Vbuffer + scanl*(256+16)+8);
	uint16 *destin = lj_curRenderingScreenPtr+scanl*(256);
	int j;
	for (j=0;j<256;j++)
	{
		*destin++=scalePal[nesppupal[*source++]];		
	}*/
	__asm{
		stmfd r13!,{r9,r10}
	
		ldr   r12, = scalePal		
		add	  r12,r12,r10

		
		mov   r3,#32
	 loop:			
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r5,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r6,[r12,r4,lsl #2]
		orr	  r5,r5,r6,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r6,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r7,[r12,r4,lsl #2]
		orr	  r6,r6,r7,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r7,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r8,[r12,r4,lsl #2]
		orr	  r7,r7,r8,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r8,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r9,[r12,r4,lsl #2]
		orr	  r8,r8,r9,lsl #16
		
		//write 8 pixels
		stmia r1!,{r5,r6,r7,r8}
		
		subs  r3,r3,#1
		bne   loop	
		
		ldmfd r13!,{r9,r10}
	}
	


}
void XlatNESBufferLineZ (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 scanl)
{
}
void XlatNESBufferLineZ16 (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 scanl)
{
}
void XlatNESBufferLineZ16B (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 scanl)
{
}
void XlatNESBufferLine16 (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 scanl)
{
}

void XlatNESBufferLineLazy (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 startscanl)
{
/*	uint8 *source = (uint8*)(src);//Vnes.var.Vbuffer + startscanl*(256+16)+8);
	uint16 *destin = lj_curRenderingScreenPtr+startscanl*(256);
	int i,j;
		
	for (i=ppuScanline-startscanl;i;i--,source+=16)
		for (j=0;j<256;j++)
		{
			*destin++=scalePal[nesppupal[*source++]];
		}  	    	*/
	__asm{
		stmfd r13!,{r9,r10}
	
		ldr   r12, = scalePal		
		add	  r12,r12,r10
		
		ldr	  r4, = ppuScanline
		ldr	  r4,[r4,r10]
		sub	  r3,r4,r3

	 loopDep:	
		mov   r10,#32
	 loop:			
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r5,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r6,[r12,r4,lsl #2]
		orr	  r5,r5,r6,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r6,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r7,[r12,r4,lsl #2]
		orr	  r6,r6,r7,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r7,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r8,[r12,r4,lsl #2]
		orr	  r7,r7,r8,lsl #16
		
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r8,[r12,r4,lsl #2]
		ldrb  r4,[r0],#1
		ldrb  r4,[r2,r4]
		ldr   r9,[r12,r4,lsl #2]
		orr	  r8,r8,r9,lsl #16
		
		//write 8 pixels
		stmia r1!,{r5,r6,r7,r8}
		
		subs  r10,r10,#1
		bne   loop
		subs  r3,r3,#1
		addne r0,r0,#16		
		bne	  loopDep
		
		ldmfd r13!,{r9,r10}	
		}	
}
void XlatNESBufferLineZLazy (unsigned char *src, unsigned char *dest,uint8 *nesppupal, uint32 startscanl)
{
}
