
#ifndef __MEM__
#define __MEM__
//=============================================================================

#define ROM_START	0x200000
#define ROM_END		0x3FFFFF

#define HIROM_START	0x800000
#define HIROM_END	0x9FFFFF

#define BIOS_START	0xFF0000
#define BIOS_END	0xFFFFFF

void reset_memory(void);
void expt_post_write(_u32 address);

void* expt_translate_address_read(_u32 address);
void* expt_translate_address_write(_u32 address);


#include "Z80_interface.h"
#include "sound.h"

extern _u8 timer[4];

__inline post_write(_u32 address)
{
/*	address &= 0xFFFFFF;

	if ((*(_u16*)(ram + 0xb8)) == 0xAA55)
	{
		expt_post_write(address);
	}
	else if (address == 0xA2)	
	{
		expt_post_write(address);
	}
	else if (address == 0xBA)
	{
		expt_post_write(address);
	}*/    
	address &= 0xFFFFFF;

	//Direct Access to Sound Chips
	if ((*(_u16*)(ram + 0xb8)) == 0xAA55)
	{
		if (address == 0xA1)	Write_SoundChipTone(ram[0xA1]);
		if (address == 0xA0)	Write_SoundChipNoise(ram[0xA0]);
	}

	//DAC Write
	if (address == 0xA2)	dac_write();

	//Clear counters?
	if (address == 0x20)
	{
		_u8 TRUN = ram[0x20];

		if ((TRUN & 0x01) == 0)		timer[0] = 0;
		if ((TRUN & 0x02) == 0)		timer[1] = 0;
		if ((TRUN & 0x04) == 0)		timer[2] = 0;
		if ((TRUN & 0x08) == 0)		timer[3] = 0;
	}

	//z80 - NMI
	if (address == 0xBA)
		Z80_nmi();    
}

extern void* translate_address_read(_u32 address);

__inline void* translate_address_write(_u32 address)
{
	address &= 0xFFFFFF;
	
	if (address <= RAM_END)
		return ram + address;
	return expt_translate_address_write( address);
}

__inline void storeB2(_u32 address, _u8 data)
{
	_u8* ptr = (_u8*)translate_address_write(address);

	//Write
	if (ptr)
	{
		*ptr = data;
		post_write(address);
	}
}

__inline void storeW2(_u32 address, _u16 data)
{

	_u8* ptr =(_u8*) translate_address_write(address);

	//Write
	if (ptr)
	{
        *ptr=(data>>0)&0xff;
        *(ptr+1)=(data>>8)&0xff;
	    post_write(address);
	}
}

__inline void storeL2(_u32 address, _u32 data)
{

	_u8* ptr =(_u8*) translate_address_write(address);

	//Write
	if (ptr)
	{
        *ptr=(data>>0)&0xff;
        *(ptr+1)=(data>>8)&0xff;
        *(ptr+2)=(data>>16)&0xff;
        *(ptr+3)=(data>>24)&0xff;
	    post_write(address);
	}
}

__inline void storeB(_u32 address, _u8 data)
{
	address &= 0xFFFFFF;
	if ( address>0xBA && address<RAM_END )
		ram[address] = data;
	else
		storeB2(address,data);
}

__inline void storeW(_u32 address, _u16 data)
{
	address &= 0xFFFFFF;
	if ( address>0xBA && address<RAM_END )
	{
		ram[address+0] = (_u8)(data>>0);
		ram[address+1] = (_u8)(data>>8);
	}
	else
		storeW2(address,data);
}

__inline void storeL(_u32 address, _u32 data)
{
	address &= 0xFFFFFF;
	if ( address>0xBA && address<RAM_END )
	{
		ram[address+0] = (_u8)(data>>0);
		ram[address+1] = (_u8)(data>>8);
		ram[address+2] = (_u8)(data>>16);
		ram[address+3] = (_u8)(data>>24);
	}
	else
		storeL2(address,data);
}


void dump_memory(_u32 start, _u32 length);

extern BOOL debug_abort_memory;
extern BOOL debug_mask_memory_error_messages;

extern BOOL memory_unlock_flash_write;
extern BOOL memory_flash_error;
extern BOOL memory_flash_command;

extern BOOL eepromStatusEnable;

//=============================================================================

extern _u8  loadB(_u32 address);
extern _u16 loadW(_u32 address);
extern _u32 loadL(_u32 address);

extern _u8* adr_trans_tab[];

/*__inline _u8  loadB2(_u32 address)
{
	_u8* ptr =(_u8*) translate_address_read(address);
	return *ptr;
}	

__inline _u16  loadW2(_u32 address)
{
	// ===================================
	if (address <= (RAM_END-1))
		 return (*(ram+address))+(*(ram+address+1)<<8);
	else
		return loadW(address);
}	

__inline _u32 loadL2(_u32 address)
{
	// ===================================
	address&=0xFFFFFF;
	if (address <= (RAM_END-3) && address!= 0x8008 )
		return *(_u32*)(_u8*)ram + address;
	else
		return loadL(address);
}*/	

//#define loadB loadB2
//#define loadW loadW2
//#define loadL loadL2

//void storeB(_u32 address, _u8 data);
//void storeW(_u32 address, _u16 data);
//void storeL(_u32 address, _u32 data);

//=============================================================================
#endif
