#include "snes9x.h"
#include "snes9x/font.h"

#include "zlib.h"

#define MAX_STATE_SIZE 512*1024
#define MAX_ZIPPED_STATE_SIZE 512*1024

int os9x_statecompress(char *source,char *dest,char *img)
{
	z_stream c_stream; /* compression stream */	
    int err;
    uLong len;    
	//Byte *compr;    
	Byte *compr;
    uint32 comprLen;
    Byte *uncompr;
    uLong uncomprLen;
    int i,j;
    char str[256];
	FILE *f;
	
	f=fopen(source,"rb");
	if (!f) return 1;
	fseek(f,-1,SEEK_END);
	uncomprLen=ftell(f)+1;
	fseek(f,0,SEEK_SET);
			
	uncompr=(Byte*)ljz_malloc(uncomprLen+60*64*2);
	if (!uncompr) {fclose(f);return 1;}
	
	
	len=fread(uncompr,1,uncomprLen,f);
	if (len!=uncomprLen) 
	{
		ljz_free(uncompr);
		fclose(f);
		return -1;
	}
	fclose(f);
	
	sprintf(str,"len %d",uncomprLen);
	menu_confirm(str);
		
	if (img)
	{
		for (i=0;i<64;i++)
		for (j=0;j<60;j++) 
		{
			uncompr[uncomprLen+((i*60+j)<<1)]=img[(32*240+i*4*240+j*4)<<1];
			uncompr[uncomprLen+(((i*60+j)<<1)|1)]=img[((32*240+i*4*240+j*4)<<1)|1];
		}
		uncomprLen+=60*64*2;
	}
		
	comprLen=MAX_ZIPPED_STATE_SIZE; //should be enough for a savestate
	compr=(Byte*)ljz_malloc(comprLen);
	if (!compr) 
	{
		ljz_free(uncompr);
		return 2;
	}
	
    c_stream.zalloc = (alloc_func)0;
    c_stream.zfree = (free_func)0;
    c_stream.opaque = (voidpf)0;
    
    c_stream.next_in  = (Bytef*)uncompr;
    c_stream.next_out = compr;
    c_stream.avail_in = (uInt)uncomprLen;
    c_stream.avail_out = (uInt)comprLen;
    
/*    sprintf(str,"mem avail : %d err %d",gm_availablesize(),err);
	MsgBold(160,10,str,0xFFFF);
	gp32_pause();*/

    err = deflateInit(&c_stream, Z_BEST_COMPRESSION/*Z_DEFAULT_COMPRESSION*/);
//    CHECK_ERR(err, "deflateInit");
	if (err) 
	{
		sprintf(str,"err %d",err);
		menu_confirm(str);
	
		ljz_free(compr);
		ljz_free(uncompr);
		return 3;
	}
	
/*	sprintf(str,"mem avail : %d err %d",gm_availablesize(),err);
	MsgBold(160,20,str,0xFFFF);
	gp32_pause();*/

/*	sprintf(str,"mem avail : %d err %d",gm_availablesize(),err);
	MsgBold(160,20,str,0xFFFF);
	gp32_pause();	    */
    
    err = deflate(&c_stream, Z_NO_FLUSH);
    //CHECK_ERR(err, "deflate");
    if (c_stream.avail_in != 0) {
      //  fprintf(stderr, "deflate not greedy\n");
		return 4;
    }
    err = deflate(&c_stream, Z_FINISH);
    if (err != Z_STREAM_END) {
    //    fprintf(stderr, "deflate should report Z_STREAM_END\n");
	//exit(1);
		return 5;
    }

    err = deflateEnd(&c_stream);
    
    //sprintf(str,"original size : %d\ncompressed size : %d",c_stream.total_in,/*i*/c_stream.total_out);
    //S9xMessage(0,0,str);
                
	ljz_free(uncompr);
			
	f=fopen(dest,"w+b");
	if (!f) 
	{
		ljz_free(compr);
		return 6;
	}
	fwrite(compr,c_stream.total_out,1,f);
	ljz_free(compr);
	fclose(f); 
	return 0;
}


int os9x_stateuncompress(char *source,char *dest)
{
	z_stream d_stream; /* compression stream */	
    int err;
    uLong len;    
	//Byte *compr;    
	Byte *compr;
    uLong comprLen;
    Byte *uncompr;
    uLong uncomprLen;
//    char str[256];
	FILE *f;
	
	//S9xMessage(0,0,"Reading file");	

	f=fopen(source,"rb");
	if (!f) return 1;
	fseek(f,-1,SEEK_END);
	comprLen=ftell(f)+1;
	fseek(f,0,SEEK_SET);
	
	compr=(Byte*)ljz_malloc(comprLen);
		
	len=fread(compr,1,comprLen,f);
	if (len!=comprLen) 
	{
		ljz_free(compr);
		fclose(f);
		return -1;
	}
	fclose(f);
	
	//S9xMessage(0,0,"Decompressing");
	
	uncompr=(Byte*)ljz_malloc(MAX_STATE_SIZE);
	uncomprLen=MAX_STATE_SIZE;


    d_stream.zalloc = (alloc_func)0;
    d_stream.zfree = (free_func)0;
    d_stream.opaque = (voidpf)0;

    d_stream.next_in  = compr;
    d_stream.avail_in = 0;
    d_stream.next_out = uncompr;

    err = inflateInit(&d_stream);
//    CHECK_ERR(err, "inflateInit");

	d_stream.avail_in = (uInt)comprLen;
    for (;;) {
        d_stream.next_out = uncompr;            /* discard the output */
		d_stream.avail_out = (uInt)uncomprLen;
        err = inflate(&d_stream, Z_NO_FLUSH);
        if (err == Z_STREAM_END) break;
        //CHECK_ERR(err, "large inflate");
    }

    err = inflateEnd(&d_stream);
  //  CHECK_ERR(err, "inflateEnd");
  
	//S9xMessage(0,0,"done");
	ljz_free(compr);
			
	f=fopen(dest,"w+b");
	if (!f) 
	{
		ljz_free(uncompr);
		return -2;
	}
	fwrite(uncompr,d_stream.total_out,1,f);	
	ljz_free(uncompr);
	fclose(f); 
	return 0;
}


/*************************************************************/

void os9x_Printc(uint32 x,uint32 y,char c,uint8 *scr,int col,int bold)
{
  int indice=c-32; //index in the font.h array
  int indicex,indicey;  
  uint32 i,j;
  int colB = (((col)&31)>>1)|((((col>>5)&63)>>1)<<5)|((((col>>11)&31)>>1)<<11);
  

  if (indice>=96) indice=0;
  if (indice<0) indice=0;
  indicey=(indice>>4)*9;
  indicex=(indice&0xf)<<3;
  
  
  
  	if (bold)
  	{
		  for (i=0;i<8;i++)
	      for (j=0;j<9;j++) 
	      {
		    if (font[j+indicey][i+indicex]=='#') 
			{
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1]=col;
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1|1]=col>>8;
			}			
			if (font[j+indicey][i+indicex]=='.')
			{
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1]=colB;
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1|1]=colB>>8;
			}
		  }
	}
	else
	{
		  for (i=0;i<8;i++)
	      for (j=0;j<9;j++) 
		    if (font[j+indicey][i+indicex]=='#') 
			{
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1]=col;
				((uint8 *)(scr))[((y+j)*320+((i+x)))<<1|1]=col>>8;
			}  	
	}
  
}

void os9x_TextOut(uint8 *buffer,int X,int Y,char *A,int col,int bold)
{
	int l;
    int k;
	int msg_x,msg_y;  		
	if (A)
	{
		l=strlen(A);
		msg_x=X;
		msg_y=Y;
		k=0;   
		while (k<l)
		{
			if (A[k]!='\n') 
			{
				os9x_Printc(msg_x,msg_y,A[k],buffer,col,bold);
				msg_x+=7;
			}	
			else 
			{
			    msg_y+=8;
				msg_x=X;
			}
			if (msg_x>=(320-7))
			{
				msg_y+=8;
				msg_x=0;
			}
			if (msg_y>=(320-8))
			{
				msg_x=0;
				msg_y=0;	      
			}
			k++;
		}
	}				
}


void os9x_BigPrintc(uint32 x,uint32 y,char c,uint8 *scr,int col,int bold)
{
  int indice=c-32; //index in the font.h array
  int indicex,indicey;  
  uint32 i,j;
  int r,g,b;
  int colB;
  r=(col>>11)&31;
  g=(col>>5)&63;
  b=col&31;
  if (bold==1)
  {
	  r+=24;if (r>31) r=31;
	  g+=48;if (g>63) g=63;
	  b+=24;if (b>31) b=31;
  }
  else
  {
	  r>>=1;g>>=1;b>>=1;

  }
  colB=(r<<11)|(g<<5)|b;
  

  if (indice>=96) indice=0;
  if (indice<0) indice=0;
  indicey=(indice>>4)*9;
  indicex=(indice&0xf)<<3;
  
  
  
  	if (bold)
  	{
		  for (i=0;i<8;i++)
	      for (j=0;j<9;j++) 
	      {
		    if (font[j+indicey][i+indicex]=='#') 
			{
				((uint16 *)(scr))[((y+j*2)*320+((i*2+x)))]=col;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+x)))]=col;
				((uint16 *)(scr))[((y+j*2)*320+((i*2+1+x)))]=col;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+1+x)))]=col;
			}			
			if (font[j+indicey][i+indicex]=='.')
			{
				((uint16 *)(scr))[((y+j*2)*320+((i*2+x)))]=colB;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+x)))]=colB;
				((uint16 *)(scr))[((y+j*2)*320+((i*2+1+x)))]=colB;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+1+x)))]=colB;
			}
		  }
	}
	else
	{
		  for (i=0;i<8;i++)
	      for (j=0;j<9;j++) 
		    if (font[j+indicey][i+indicex]=='#') 
			{
				((uint16 *)(scr))[((y+j*2)*320+((i*2+x)))]=col;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+x)))]=col;
				((uint16 *)(scr))[((y+j*2)*320+((i*2+1+x)))]=col;
				((uint16 *)(scr))[((y+j*2+1)*320+((i*2+1+x)))]=col;
			}  	
	}
  
}

void os9x_BigTextOut(uint8 *buffer,int X,int Y,char *A,int col,int bold)
{
	int l;
    int k;
	int msg_x,msg_y;  		
	if (A)
	{
		l=strlen(A);
		msg_x=X;
		msg_y=Y;
		k=0;   
		while (k<l)
		{
			if (A[k]!='\n') 
			{
				os9x_BigPrintc(msg_x,msg_y,A[k],buffer,col,bold);
				msg_x+=7*2;
			}	
			else 
			{
			    msg_y+=8*2;
				msg_x=X;
			}
			if (msg_x>=(320-7*2))
			{
				msg_y+=8*2;
				msg_x=0;
			}
			if (msg_y>=(320-8))
			{
				msg_x=0;
				msg_y=0;	      
			}
			k++;
		}
	}				
}


void os9x_Button(uint8 *buffer,int X,int Y,char *A)
{
	int max_x,max_y,x,y;
	int i,j;
	if (!A) return;
	max_x=max_y=x=0;	
	i=0;
	while (A[i])
	{
		if (A[i]!='\n')
		{
			x+=7*2;
		}
		else
		{
			if (x>max_x) max_x=x;
			max_y+=8*2;
			x=0;
		}
		i++;
	}
	if (x>max_x) max_x=x;
	max_y+=8*2;
	
	max_x+=6;
	max_y+=6;
	
	for (y=Y;y<Y+max_y;y++)
	for (x=X;x<X+max_x;x++)
	{
		((uint16*)buffer)[y*320+x]=0xFFFF;
	}
	
	for (y=0;y<2;y++)
	for (x=X;x<X+max_x;x++)
	{
		((uint16*)buffer)[(Y+y)*320+x]=(24<<11)|(48<<5)|24;
		((uint16*)buffer)[(Y+max_y-y-1)*320+x]=(24<<11)|(48<<5)|24;
	}
	
	for (y=Y;y<Y+max_y;y++)
	for (x=0;x<2;x++)
	{
		((uint16*)buffer)[y*320+X+x]=(24<<11)|(48<<5)|24;
		((uint16*)buffer)[y*320+X+max_x-x-1]=(24<<11)|(48<<5)|24;
	}
	
	
	os9x_BigTextOut(buffer,X+3,Y+3,A,(8<<11)|(10<<5)|15,1);
}

