///////////////////////////////////////////////////////////////////////////////
// Wonderswan emulator
////////////////////////////////////////////////////////////////////////////////
//
// 13.04.2002: Fixed a small bug causing crashes
//
//
//
//
//////////////////////////////////////////////////////////////////////////////

#define OSWANZ_TURBOFSKIP 15
#define OSWANZ_ROMSDIR "0:/PALM/Programs/LJZ/WS/Roms/"
#define OSWANZ_SAVESDIR "0:/PALM/Programs/LJZ/WS/Saves/"
#define AUTO_FSKIP 10
#define SKEEZIX_FSKIP 11

#define MAX_FRAME_SKIPPED 10

#define KEY_UP    	0x00000001
#define KEY_DOWN  	0x00000002
#define KEY_LEFT  	0x00000004
#define KEY_RIGHT 	0x00000008
#define KEY_SEL   	0x00000010
#define KEY_A     	0x00000020
#define KEY_B	  	0x00000040
#define KEY_START	0x00000080
#define KEY_FLIPPED	0x00000100
#define KEY_QUICKLOAD   0x00000200
#define KEY_QUICKSAVE   0x00000400
#define KEY_TURBO  	0x00000800
#define KEY_SCREENMODE	0x00001000
#define KEY_FRAMESKIP	0x00002000


#include <TapWave.h>

using namespace std;
extern "C" {
#include "LJZmenu.h"
}


#include "ljz_std.h"

#undef OF


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <malloc.h>
//#include <io.h>
#include <fcntl.h> 
#include <time.h>

#include "log.h"
#include "rom.h"
#include "nec/nec.h"
#include "memory.h"
#include "gpu.h"
#include "io.h"
#include "ws.h"


extern "C" int __getdate_file(const char* name);


// Assertion Support
static void
assert_check(int error, const char* message)
{
    if (error) {
        ErrDisplayFileLineMsg("PalmOS", (UInt16) error, message);
    }
}

#define xassert(error) {assert_check(error, #error);}

static void showError(const char* aWhat, Err aError) {
    ErrDisplayFileLineMsg("oink", (UInt16) aError, aWhat);
}

static void showError(const char* aWhat, UInt32 aValue) {
    char chbuf[50];
    StrPrintF(chbuf, "%lx", aValue);
    ErrDisplayFileLineMsg(aWhat, (UInt16)0, chbuf);
}


////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
//
//
//
//
//
//
//
////////////////////////////////////////////////////////////////////////////////
#define		LOG_PATH "/PALM/Programs/LJZ/WS/wslog.log"

char str_month[12][4]=
{"Jan","Feb","Mar","Apr","May","Jun",
"Jul","Aug","Sep","Oct","Nov","Dec"};


int			app_gameRunning=0;
int			app_terminate=0;


int			app_rotated=0;


int			ws_videoEnhancementType=0;
int			ws_colourScheme=COLOUR_SCHEME_DEFAULT;
int			ws_system=WS_SYSTEM_COLOR;//WS_SYSTEM_AUTODETECT;


extern uint8 *ws_rom;
extern uint32  romSize;


char shortrom_filename[64];
char oswanZ_curVolume;
char RomPath[256];

void oswan_updateInput(void);
int oswanZ_loadrom();
void oswanZ_end();
int oswanZ_setupenv(TwGfxType* mGfxLib);
void oswanZ_changefskip(void);
void oswanZ_changerendering(void);
void oswanZ_updatevpad(unsigned int val);
void oswanZ_MainLoop(void);
void oswanZ_pause(int pause);
int oswanZ_ShowMenu(void);
void oswanZ_updateScreen(void);
int32 oswanZ_getTickCount(void);

TwGfxType *oswanZ_mGfxLib;
TwGfxSurfaceType *oswanZ_curRenderingScreen,*oswanZ_RenderingScreen[2];
uint16 *oswanZ_curRenderingScreenPtr,*oswanZ_RenderingScreenPtr[2];
uint32 oswanZ_curRenderingScreenIndex;
uint32 oswanZ_enabledSound,oswanZ_sndfreq;
uint32 oswanZ_renderingmode;
uint32 oswanZ_padState;
int32 oswanZ_frameskipped,oswanZ_frameskip,oswanZ_fastforward;
int32 oswanZ_frameBeginTime;
int32 oswanZ_speedpct;
int32 oswanZ_autofireA,oswanZ_autofireB;
int32 oswanZ_framecount,oswanZ_drawframe;

int32 oswanZ_frameTime;

TwInputHandle oswanZ_mInputQueue;
long oswanZ_inputFormats[] = { twInputTimeStamp };
typedef struct {long mTime;} oswanZ_ZUserEvent;
oswanZ_ZUserEvent oswanZ_mUserEvent;
long* oswanZ_mInputFormat;        

//#include "filters/standard.h"
int16 backbuffer[224*144];


#include "SoundMgr.h"
SndStreamRef snd_stream;
typedef struct {uint32 __r9;uint32 __r10;} reg_sys_t;
static reg_sys_t reg_sys;
uint32 oswanZ_sndbuffsize;
int32 snd_cb_running;

static Err oswanZ_sndCallBack (void *userData,SndStreamRef stream, void *bufferZ,UInt32 frameCountZ)
{
	__asm 
	{
		stmfd  r13!,{r9,r10}
		ldr	   r9,[r0]
		ldr	   r10,[r0,#4]
	}		
	if (snd_cb_running==0)
	{
		snd_cb_running=1;
//		sound_update((int16*)(bufferZ),frameCountZ);
		snd_cb_running=0;
	}
	
	__asm 
	{
		ldmfd  r13!,{r9,r10}
	}
	return errNone;
}

char 	savename[255];

void getstate_filename(int num,char *filename)
{
	char result[256];
	uint32 i=0,j=255,k=255,l=255;
	i=0;
	j=255;
	k=-1;
	while (RomPath[i]!=0) 
	{
		if (RomPath[i]=='.') j=i;
		if (RomPath[i]=='/') k=i;
		if (RomPath[i]=='\\') k=i;
		i++;
	  }
	if (j==255) j=i;  //pas d'extension	  
	memcpy(result,RomPath+k+1,j+1-k-1);    
	result[j+1-k-1]=0;
	strcat(result,"sw");
	sprintf(filename,"%s%s%d",OSWANZ_SAVESDIR,result,num);
	filename[0]=oswanZ_curVolume+'0';
}


int state_load(int num,int test_only)
{
	FILE	*file;			
	getstate_filename(num,savename);
	file = fopen(savename, "rb");
	if (file == NULL)
	{
		return -1;
	}
	fclose(file);
	if (test_only) return 0;
	ws_loadState(savename);
	return 0;
}

int state_save(int num)
{
	getstate_filename(num,savename);	
	ws_saveState(savename);	
	return 0;
}

int init_video(void)
{
	//Init vid
	TwGfxSurfaceInfoType surfInfo;
	surfInfo.size = sizeof(surfInfo);
	surfInfo.width = 256; //must be / 4
	surfInfo.height = 256;
	surfInfo.location = twGfxLocationAcceleratorMemoryNoBackingStore;
	surfInfo.pixelFormat = twGfxPixelFormatRGB565;
	xassert(TwGfxAllocSurface(oswanZ_mGfxLib, &oswanZ_RenderingScreen[0], &surfInfo));
	xassert(TwGfxAllocSurface(oswanZ_mGfxLib, &oswanZ_RenderingScreen[1], &surfInfo));

	if (TwGfxLockSurface(oswanZ_RenderingScreen[0], (void**)&oswanZ_RenderingScreenPtr[0]))
	{
	   return 1;
	}
	if (TwGfxLockSurface(oswanZ_RenderingScreen[1], (void**)&oswanZ_RenderingScreenPtr[1]))
	{
	   return 1;
	}
	
	memset(oswanZ_RenderingScreenPtr[0],0,256*256*2);
	memset(oswanZ_RenderingScreenPtr[1],0,256*256*2);
	
	oswanZ_curRenderingScreenIndex=0;
	oswanZ_curRenderingScreen=oswanZ_RenderingScreen[oswanZ_curRenderingScreenIndex];
	oswanZ_curRenderingScreenPtr=oswanZ_RenderingScreenPtr[oswanZ_curRenderingScreenIndex];
					
    /*input*/
    TwInputOpen(&oswanZ_mInputQueue, "twinput", "r");
    oswanZ_mInputFormat = (long*)ljz_malloc(sizeof(long)*sizeof(oswanZ_inputFormats)/sizeof(oswanZ_inputFormats[0]));
    memmove(oswanZ_mInputFormat, oswanZ_inputFormats, sizeof(oswanZ_inputFormats));
    TwInputSetFormat(oswanZ_mInputQueue, oswanZ_mInputFormat, sizeof(oswanZ_inputFormats));    
    TwInputActivate(oswanZ_mInputQueue);

	return 0;
}

void oswanZ_initparam(void)
{
	signed long val1,val2,val;
	char *ftrptr;
	FtrGet(APPLI_ID,FTR_OPT_SOUND_ID,(unsigned long*)&val);  
	oswanZ_enabledSound=val;
	FtrGet(APPLI_ID,FTR_OPT_SOUNDFREQ_ID,(unsigned long*)&val);  
	if (val==1) oswanZ_sndfreq=22050;
	else if (val==2) oswanZ_sndfreq=11025;
	else oswanZ_sndfreq=44100;	
	
	
	FtrGet(APPLI_ID,FTR_OPT_SCREENMODE_ID,(unsigned long*)&val1);
	FtrGet(APPLI_ID,FTR_OPT_SMOOTHING_ID,(unsigned long*)&val2);
		if (!val1) oswanZ_renderingmode=0;
		else 
		if (val1==1) oswanZ_renderingmode=(val2?2:1);
		else
		if (val1==2) oswanZ_renderingmode=(val2?4:3);
		else
	oswanZ_renderingmode=(val2?6:5);	

	oswanZ_fastforward=0;	
	oswanZ_frameskipped=0;
	
    FtrGet(APPLI_ID,FTR_OPT_FSKIP_ID,(unsigned long*)&val); 
	oswanZ_frameskip=val;


//Core settings
		
	oswanZ_speedpct=100;
	oswanZ_autofireA=0;
	oswanZ_autofireB=0;	
	oswanZ_framecount=0;
	oswanZ_drawframe=1;
	init_video();	
    

    strcpy(RomPath,OSWANZ_ROMSDIR);
    FtrGet(APPLI_ID,FTR_ROM_FILENAME_ID ,(unsigned long*)&ftrptr);
    
    oswanZ_curVolume=ftrptr[0];
    RomPath[0]=oswanZ_curVolume+'0';
    
    ftrptr+=3;
    strcpy(shortrom_filename,ftrptr);  
    strcat(RomPath,ftrptr);
    FtrPtrFree(APPLI_ID, FTR_ROM_FILENAME_ID);	


   
    
    oswanZ_frameTime=17; //hardcoded to NTSC
}

void oswanZ_audio_init()
{
}

void oswanZ_initSystem(void)
{     	
	app_rotated=ws_rotated();
	app_gameRunning=1;
	ws_set_system(ws_system);
	ws_set_colour_scheme(ws_colourScheme);
	//ws_buildHalfBrightnessTable();
	ws_reset();
}
	
int oswanZ_loadrom()
{	
	ljz_std_showstat(); 
	
	char strload[256];
	oswanZ_initparam();    			
	
	/* Load game */
    sprintf(strload,"Loading : %s",shortrom_filename);
    menu_message(strload);          		   
//	unsigned long freeM,maxM;	
//	MemHeapFreeBytes(0,&freeM,&maxM);
//	StrPrintF(strload,"mem : %d %d",freeM,maxM);
//	menu_message(strload);
//	menu_waitPen(NULL,NULL,1);


    if(!ws_init(RomPath))
	{
		menu_message("Error loading file");
		menu_waitPen(NULL,NULL,1);
		return 1;
	}	
		
	oswanZ_initSystem();		
    oswanZ_audio_init();            

    oswanZ_frameBeginTime=oswanZ_getTickCount();
    if (oswanZ_enabledSound) SndStreamStart(snd_stream);
      

    return 0;  
}
void oswanZ_end()
{
	int val;
	
	TwGfxSurfaceType* displaySurface;
	// Get palmos display surface:
	xassert(TwGfxGetPalmDisplaySurface(oswanZ_mGfxLib, &displaySurface));
	while (TwGfxIsSurfaceReady(displaySurface));	
	oswanZ_pause(1);
	
	

	TwGfxUnlockSurface(oswanZ_RenderingScreen[0],0);
	TwGfxUnlockSurface(oswanZ_RenderingScreen[1],0);
	if (oswanZ_RenderingScreen[0]) TwGfxFreeSurface(oswanZ_RenderingScreen[0]);
	if (oswanZ_RenderingScreen[1]) TwGfxFreeSurface(oswanZ_RenderingScreen[1]);  
	TwInputClose(oswanZ_mInputQueue);  	
	ljz_free(oswanZ_mInputFormat);    
	
	SndStreamDelete(snd_stream);  
/*emu*/
	ws_done();	

	

	
	  
 	if ((oswanZ_renderingmode)&&(!(oswanZ_renderingmode&1))) FtrSet(APPLI_ID, FTR_OPT_SMOOTHING_ID,1);
	else FtrSet(APPLI_ID, FTR_OPT_SMOOTHING_ID,0);	

	FtrSet(APPLI_ID, FTR_OPT_SOUND_ID,oswanZ_enabledSound);
	switch (oswanZ_sndfreq)
	{
		case 11025:val=2;break;
		case 22050:val=1;break;
		default:val=0;break;
	}
	FtrSet(APPLI_ID, FTR_OPT_SOUNDFREQ_ID,val);
	
	FtrSet(APPLI_ID, FTR_OPT_SCREENMODE_ID,((oswanZ_renderingmode+1)>>1));
	
	FtrSet(APPLI_ID, FTR_OPT_FSKIP_ID,oswanZ_frameskip);

	ljz_std_showstat(); 
}
int oswanZ_setupenv(TwGfxType* mGfxLib)
{
	oswanZ_mGfxLib=mGfxLib;
	return 0;
}


void oswanZ_MainLoop(void)
{
	oswanZ_framecount++;
//Throttle speed
	int haswaited;
	oswanZ_frameBeginTime+=oswanZ_frameTime*100/oswanZ_speedpct;	//60Hz
			

	//Frameskip
	if ((oswanZ_frameskip==AUTO_FSKIP)&&(!oswanZ_fastforward))
	{
		
		//do avoid issue when Tick is looping	
		if ((int32)(oswanZ_frameBeginTime-oswanZ_getTickCount())>=(int32)(20*oswanZ_frameTime))	
			oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;				
		//while (oswanZ_getTickCount()<=oswanZ_frameBeginTime)  haswaited=1;
		int i=oswanZ_frameBeginTime-oswanZ_getTickCount()+1;
        if (i>0) {haswaited=1; SysTaskDelay(i);}
		

	
		if (haswaited) 
		{
			oswanZ_frameskipped=0;
			oswanZ_drawframe=1;
		}
		else 
		{
			oswanZ_frameskipped++;						
			
			if (oswanZ_frameskipped>MAX_FRAME_SKIPPED) 
			{
				oswanZ_frameskipped=0;
				oswanZ_drawframe=1;
			}
			else oswanZ_drawframe=0;
		}			
	}
	else
	{
		if (oswanZ_frameskip==SKEEZIX_FSKIP) oswanZ_frameskipped=SKEEZIX_FSKIP;
		oswanZ_frameskipped++;
		
		if (oswanZ_fastforward)
		{
			if (oswanZ_frameskipped>OSWANZ_TURBOFSKIP)
			{
				oswanZ_frameskipped=0;
				oswanZ_drawframe=1;
			}
			else oswanZ_drawframe=0;
		}
		else
		{								
			if (oswanZ_frameskipped>oswanZ_frameskip)
			{
				oswanZ_frameskipped=0;
				oswanZ_drawframe=1;
			}
			else oswanZ_drawframe=0;
		}
		
		if ((!oswanZ_fastforward)&&(oswanZ_drawframe||(oswanZ_enabledSound==2)))
		{
			//do avoid issue when Tick is looping	
			if ((int32)(oswanZ_frameBeginTime-oswanZ_getTickCount())>=(int32)(20*oswanZ_frameTime))	
				oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;				
			//while (oswanZ_getTickCount()<=oswanZ_frameBeginTime) ;
			int i=oswanZ_frameBeginTime-oswanZ_getTickCount()+1;
        	if (i>0) {SysTaskDelay(i);}
		}

	}
	
	if (oswanZ_drawframe)
	{
		while (!ws_executeLine(backbuffer,1));		
		oswanZ_updateScreen();
	}
	else
	{
		while (!ws_executeLine(backbuffer,0));
	}
	
	oswan_updateInput();
}



void oswanZ_updateScreen(void)
{	 
//	if (oswanZ_drawframe)
	{
		TwGfxPointType destLocation = {0, 0};
	    TwGfxSurfaceType* displaySurface;
		TwGfxRectType mDisp;
		int width = 224;//(reg[12] & 1) ? 320 : 256;
	    int height = 144;//(reg[1] & 8) ? 240 : 224;
	    
		mDisp.x=0;
		mDisp.y=0;
		mDisp.w=width;
		mDisp.h=height;
    	// Get palmos display surface:
	    xassert(TwGfxGetPalmDisplaySurface(oswanZ_mGfxLib, &displaySurface));	
    
    	//if (width==256) oswanZ_asmfastrender256((uint8*)oswanZ_curRenderingScreenPtr,(uint8*)VideoBuffer,height,bitmap.viewport.x<<1);
    	//if (width==320)	oswanZ_asmfastrender320((uint8*)oswanZ_curRenderingScreenPtr,(uint8*)VideoBuffer,height,bitmap.viewport.x<<1);
    	uint16 *dest=oswanZ_curRenderingScreenPtr;
    	uint16 *src=(uint16*)backbuffer;
    	int r,g,b,col;    	
    	for (int y = 0; y < 144; y++)
		{
			for (int x = 0; x < 224; x++)
			{
				*dest++=*src++;
			}
			dest+=256-224;
		}

		switch (oswanZ_renderingmode)
		{    
		case 0:
		{						
			destLocation.x=(480-mDisp.w)>>1;
			destLocation.y=(320-mDisp.h)>>1;
			if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))			
				xassert(TwGfxBitBlt(displaySurface, &destLocation, oswanZ_curRenderingScreen, &mDisp));									
			break;
		}
		case 1:
		{
				TwGfxRectType mDispDest;
				mDispDest.w=width*320/height;
				mDispDest.h=320;				
				mDispDest.x=(480-mDispDest.w)>>1;
				mDispDest.y=0;				
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}
		case 2:
		{
				TwGfxRectType mDispDest;
				mDispDest.w=width*320/height;
				mDispDest.h=320;				
				mDispDest.x=(480-mDispDest.w)>>1;
				mDispDest.y=0;	
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}
			
	
		case 3:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}							
		case 4:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=0;
				mDispDest.y=0;
				mDispDest.w=480;
				mDispDest.h=320;
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}							
			
		case 5:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-width*2)>>1;
				mDispDest.y=(320-height*2)>>1;
				mDispDest.w=width*2;
				mDispDest.h=height*2;
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast));
				break;
			}	
		case 6:
			{
				TwGfxRectType mDispDest;
				mDispDest.x=(480-width*2)>>1;
				mDispDest.y=(320-height*2)>>1;
				mDispDest.w=width*2;
				mDispDest.h=height*2;
				if (!TwGfxIsSurfaceReady(oswanZ_curRenderingScreen))
					xassert(TwGfxStretchBlt2(displaySurface, &mDispDest, oswanZ_curRenderingScreen, &mDisp,twGfxStretchFast|twGfxStretchSmooth));
				break;
			}		
		}
		
//		oswanZ_curRenderingScreenIndex^=1;
//		oswanZ_curRenderingScreenPtr=oswanZ_RenderingScreenPtr[oswanZ_curRenderingScreenIndex];
//		bitmap.data   = (unsigned char *)oswanZ_curRenderingScreenPtr;

	}
	
	//frames_displayed++;	

}



int32 oswanZ_getTickCount(void)
{
	//TODO return tick 
	/*long long l;
	l=TimGetTicks();	
	return l*1000/SysTicksPerSecond();*/	
  	TwInputPoll(oswanZ_mInputQueue, (long *) &oswanZ_mUserEvent, sizeof(oswanZ_mUserEvent));
	return oswanZ_mUserEvent.mTime;
}


void oswanZ_pause(int pause)
{	
   if (pause)
	{
		if (oswanZ_enabledSound) SndStreamStop(snd_stream);
	}
	else
	{
		oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
		if (oswanZ_enabledSound) SndStreamStart(snd_stream);		
	}
}


void oswanZ_changefskip(void)
{
	char tmp[64];
	int i;
	

	oswanZ_frameskip++;
	if (oswanZ_frameskip>SKEEZIX_FSKIP) oswanZ_frameskip=0;
	
	
	if (oswanZ_frameskip==AUTO_FSKIP) sprintf(tmp,"Frameskip : AUTO");
	else if (oswanZ_frameskip==SKEEZIX_FSKIP) sprintf(tmp,"Frameskip : Skeezix");
	else sprintf(tmp,"Frameskip : %d",oswanZ_frameskip);	
	//DrawMessage(tmp,0);
	menu_message(tmp);
	
	i=oswanZ_getTickCount();
	while ((oswanZ_getTickCount()-i)<300)
	{
	}
	oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
}
void oswanZ_changerendering(void)
{	
	oswanZ_renderingmode++;
	if (oswanZ_renderingmode==7) oswanZ_renderingmode=0;

	menu_message("Changing screen mode");
	oswanZ_frameBeginTime=oswanZ_getTickCount();
	while ((oswanZ_getTickCount()-oswanZ_frameBeginTime)<300)
	{
	}
	oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
}


void oswanZ_updatevpad(unsigned int val)
{
	oswanZ_padState=val;
}

/****************************************************************************/

int oswanZ_ShowMenuGFXConfig(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  int scrmode,smoothing,fskipval;
  RectangleType *item_rect;
  item_rect=(RectangleType*)ljz_malloc(12*sizeof(RectangleType));
  char tmp[16];
  

	
  if (!oswanZ_renderingmode) {scrmode=0;smoothing=0;}
  else {scrmode=(oswanZ_renderingmode+1)>>1;smoothing=!(oswanZ_renderingmode&1);}
	
  fskipval=oswanZ_frameskip;
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr) memcpy(scr,savescr,480*320*2);	    
		//if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
		
		menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);
	  
	  	menu_button(10,80,(char*)"FRAMESKIP",0,&item_rect[1],2);	  
	  	menu_bar(200,80,fskipval,11,&item_rect[2]);
	  	if (fskipval<AUTO_FSKIP) sprintf(tmp,"%d",fskipval);
	  	else if (fskipval==AUTO_FSKIP) sprintf(tmp,"AUTO");
	  	else sprintf(tmp,"Skeezix");
	  	menu_messageRaw(320,80,tmp);
	  
	  	menu_button(10,130,(char*)"SCREENMODE",0,&item_rect[3],2);
	  	menu_button(200,130,(char*)"1:1",(scrmode==0?2:1),&item_rect[4],0);
	  	menu_button(250,130,(char*)"4:3",(scrmode==1?2:1),&item_rect[5],0);
	  	menu_button(300,130,(char*)"FULL",(scrmode==2?2:1),&item_rect[6],0);
	  	menu_button(350,130,(char*)"ZOOM",(scrmode==3?2:1),&item_rect[7],1);
	  
	  	menu_button(10,180,(char*)"SMOOTHING",0,&item_rect[8],2);
	  	menu_button(200,180,(char*)"OFF",(smoothing?1:2),&item_rect[9],0);
	  	menu_button(250,180,(char*)"ON",(smoothing?2:1),&item_rect[10],0);	  	  	  
	  	
	  	if (scr) WinScreenUnlock();
	
  	  }
	  menu_waitPen(&x,&y,0);
	  
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}
	  //FSKIP
	  if (TEST_PEN_MENU(x,y,item_rect[2])) 
	  {
	  	fskipval=(x-10-item_rect[2].topLeft.x)*11/100;
	  	if (fskipval<0) fskipval=0;
	  	if (fskipval>11) fskipval=11;
	  	redraw=1;
	  }
	  //SCREEN MODES
	  if (TEST_PEN_MENU(x,y,item_rect[4])) 
	  {
	  	  scrmode=0;
	  	  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[5])) 
	  {
	  	  scrmode=1;
	  	  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[6])) 
	  {
	  	  scrmode=2;
	  	  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[7])) 
	  {
	  	  scrmode=3;
	  	  redraw=1;
	  }
	  //SMOOTHING
	  if (TEST_PEN_MENU(x,y,item_rect[9])&&scrmode) 
	  {
	  	  smoothing=0;
	  	  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[10])&&scrmode) 
	  {
	  	  smoothing=1;
	  	  redraw=1;
	  }
  }  
  
  oswanZ_frameskip=fskipval;

  if (!scrmode) oswanZ_renderingmode=0;
	else 
	if (scrmode==1) oswanZ_renderingmode=(smoothing?2:1);
	else
	if (scrmode==2) oswanZ_renderingmode=(smoothing?4:3);
	else
	oswanZ_renderingmode=(smoothing?6:5);
	
  menu_waitPen(NULL,NULL,2);
  
  ljz_free(item_rect);
  return result;
}



int oswanZ_ShowMenuSNDConfig(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  int vol;
  int bassboost;
  RectangleType *item_rect;
  char tmp[16];
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));
  
  vol=TwSndGetVolume();
  bassboost=TwSndGetBassBoost();
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr) memcpy(scr,savescr,480*320*2);	    
//		if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  	
  		
  	  	  	  	  	  	
	  menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);	  
	  menu_button(10,80,(char*)"SOUND",0,&item_rect[1],2);
	  menu_button(200,80,(char*)"OFF",(oswanZ_enabledSound==0?2:1),&item_rect[2],0);
	  menu_button(250,80,(char*)"NOZ80",(oswanZ_enabledSound==1?2:1),&item_rect[3],0);	  
	  menu_button(300,80,(char*)"FULL",(oswanZ_enabledSound==2?2:1),&item_rect[4],0);	 
	  
/*	  menu_button(10,120,(char*)"VOLUME",0,&item_rect[5],2);	  
	  menu_bar(200,120,vol,64,&item_rect[6]);
	  sprintf(tmp,"%d%",vol*100/64);	  	
	  	menu_messageRaw(320,120,tmp);*/
	  
	  menu_button(10,160,(char*)"BASS BOOST",0,&item_rect[7],2);
	  menu_button(200,160,(char*)"OFF",(bassboost==0?2:1),&item_rect[8],0);
	  menu_button(300,160,(char*)"ON",(bassboost?2:1),&item_rect[9],0);	 
		if (scr) WinScreenUnlock();	  
  	  }
	  menu_waitPen(&x,&y,0);
	  
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}	  
	  //SOUND OFF / z80 off
	  if (TEST_PEN_MENU(x,y,item_rect[2])) 
	  {
		  oswanZ_enabledSound=0;
		  redraw=1;
	  }
	  //SOUND OFF / z80 on
	  if (TEST_PEN_MENU(x,y,item_rect[3])) 
	  {
	  	  if (!oswanZ_enabledSound) 
	  	  {
	  	  	if (menu_confirm("This will need to reset WS"))
	  	  	{
	  	  		oswanZ_enabledSound=1;

	  	  		ws_reset();
	  	  	}
	  	  }
	  	  else
	  	  {
		  	    oswanZ_enabledSound=1;

	  	  }
	  	  redraw=1;
	  }
	  //SOUND ON / z80 on
	  if (TEST_PEN_MENU(x,y,item_rect[4])) 
	  {
	  	  if (!oswanZ_enabledSound)
	  	  {
	  	  	if (menu_confirm("This will need to reset WS"))
	  	  	{
	  	  		oswanZ_enabledSound=1;

	  	  		ws_reset();
	  	  	}
	  	  }
	  	  else
			  oswanZ_enabledSound=2;
		  redraw=1;
	  }
	  
	  //VOLUME
/*	  if (TEST_PEN_MENU(x,y,item_rect[6])) 
	  {
	  	vol=(x-item_rect[6].topLeft.x)*64/100;
	  	if (vol>64) vol=64;
	  	redraw=1;
	  }*/
	  //BASSBOOST OFF
	  if (TEST_PEN_MENU(x,y,item_rect[8])) 
	  {
	  	bassboost=0;
	  	redraw=1;
	  }
	  //BASSBOOST ON
	  if (TEST_PEN_MENU(x,y,item_rect[9])) 
	  {
	  	bassboost=1;
	  	redraw=1;
	  }
  }  
  
  TwSndSetVolume(vol);
  TwSndSetBassBoost(bassboost);
  
  menu_waitPen(NULL,NULL,2);
  
  ljz_free(item_rect);
  return result;
}


int oswanZ_ShowMenuOTHERConfig(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  RectangleType *item_rect;
  char tmp[16];
  item_rect=(RectangleType*)ljz_malloc(20*sizeof(RectangleType));
  
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr) memcpy(scr,savescr,480*320*2);	    
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  	
  		
  	  	  	  	  	  	
	  menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);	  
	  menu_button(10,80,(char*)"AUTOFIRE A",0,&item_rect[1],2);
	  menu_button(200,80,(char*)"OFF",(oswanZ_autofireA==0?2:1),&item_rect[2],0);
	  menu_button(250,80,(char*)"SLW",(oswanZ_autofireA==1?2:1),&item_rect[3],0);
	  menu_button(300,80,(char*)"MED",(oswanZ_autofireA==2?2:1),&item_rect[4],0);
	  menu_button(350,80,(char*)"FST",(oswanZ_autofireA==3?2:1),&item_rect[5],0);
	  
	  menu_button(10,120,(char*)"AUTOFIRE B",0,&item_rect[6],2);
	  menu_button(200,120,(char*)"OFF",(oswanZ_autofireB==0?2:1),&item_rect[7],0);
	  menu_button(250,120,(char*)"SLW",(oswanZ_autofireB==1?2:1),&item_rect[8],0);
	  menu_button(300,120,(char*)"MED",(oswanZ_autofireB==2?2:1),&item_rect[9],0);
	  menu_button(350,120,(char*)"FST",(oswanZ_autofireB==3?2:1),&item_rect[10],0);
	  
	  
	  menu_button(10,160,(char*)"EMU SPEED",0,&item_rect[11],2);	  
	  menu_bar(200,160,oswanZ_speedpct,100,&item_rect[12]);
	  sprintf(tmp,"%d%",oswanZ_speedpct);
    	menu_messageRaw(320,160,tmp);
	  
		if (scr) WinScreenUnlock();	  
  	  }
	  menu_waitPen(&x,&y,0);
	  
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}	  
	  //AUTOFIRE A
	  if (TEST_PEN_MENU(x,y,item_rect[2])) 
	  {
		  oswanZ_autofireA=0;		  
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[3])) 
	  {
		  oswanZ_autofireA=1;
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[4])) 
	  {
		  oswanZ_autofireA=2;
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[5])) 
	  {
		  oswanZ_autofireA=3;
		  redraw=1;
	  }
	  //AUTOFIRE B
	  if (TEST_PEN_MENU(x,y,item_rect[7])) 
	  {
		  oswanZ_autofireB=0;		  
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[8])) 
	  {
		  oswanZ_autofireB=1;
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[9])) 
	  {
		  oswanZ_autofireB=2;
		  redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[10])) 
	  {
		  oswanZ_autofireB=3;
		  redraw=1;
	  }
	  //SPEED
	  if (TEST_PEN_MENU(x,y,item_rect[12])) 
	  {
	  	oswanZ_speedpct=(x-10-item_rect[12].topLeft.x);
	  	if (oswanZ_speedpct>100) oswanZ_speedpct=100;
	  	if (oswanZ_speedpct<10) oswanZ_speedpct=10;
	  	redraw=1;
	  }
  }  
  
  menu_waitPen(NULL,NULL,2);
  
  ljz_free(item_rect);
  return result;
}


int oswanZ_ShowMenuConfig(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  RectangleType *item_rect;
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr&&savescr) memcpy(scr,savescr,480*320*2);	    
		//if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  	
  		
  	  	  	  	  	  	
	  menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);	  
	  menu_button(172,100,(char*)"GFX",0,&item_rect[1],2);
	  //menu_button(172,150,(char*)"SOUND",0,&item_rect[2],2);
	  menu_button(172,200,(char*)"OTHER",0,&item_rect[3],2);
	  menu_button(172,250,(char*)"SND & BRT",9,&item_rect[4],2);
	  if (scr) WinScreenUnlock();
  	  }
	  menu_waitPen(&x,&y,1);
	  
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}	  
	  //GFX
	  if (TEST_PEN_MENU(x,y,item_rect[1])) 
	  {
	  	oswanZ_ShowMenuGFXConfig(savescr);
		redraw=1;
	  }
	  //SOUND
/*	  if (TEST_PEN_MENU(x,y,item_rect[2])) 
	  {
     	oswanZ_ShowMenuSNDConfig(savescr);
		redraw=1;
	  }*/
	  //OTHER
	  if (TEST_PEN_MENU(x,y,item_rect[3])) 
	  {
		  oswanZ_ShowMenuOTHERConfig(savescr);
		  redraw=1;		  
	  }
	  //SND & BRT
	  if (TEST_PEN_MENU(x,y,item_rect[4]))
	  {
	  	  EventType myevent;
	  	  
	  	  for (;;)	  	  
	  	  {
		  	  EvtGetEvent(&myevent,0);
	  	  	  if (myevent.eType==nilEvent) break;			  
		  }
	  	  
	  	  myevent.eType=keyDownEvent;
	  	  myevent.data.keyDown.chr=vchrBrightness;	  	  
	  	  EvtAddEventToQueue(&myevent);
	  	  
	  	  for (;;)	  	  
	  	  {
		  	  EvtGetEvent(&myevent,100);
	  	  	  if (myevent.eType==nilEvent) break;
			  if (myevent.eType==keyDownEvent)
			  {
			  	if (myevent.data.keyDown.chr!=vchrLaunch)
			  	{SysHandleEvent(&myevent);}
			  }			  
		  }
		  redraw=1;
	  }
  }  
  ljz_free(item_rect);
  return result;
}



int oswanZ_ShowMenuSave(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  RectangleType *item_rect;
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));
  memset((char*)item_rect,0,15*sizeof(RectangleType));

   int slot[4];
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr&&savescr) memcpy(scr,savescr,480*320*2);	    
		//if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  	
  		
  	  	  	  	  	  	
	  FILE *savefile;
  	 
  	  slot[0]=slot[1]=slot[2]=slot[3]=0;
  	  for (int i=0;i<4;i++) 
  	  	if (!state_load(i,1))
  	  	{
      		slot[i]=1;
      
	  		struct tm *localTime;
			time_t long_time;
			char str[255];		
			getstate_filename(i,str);
			long_time=__getdate_file(str);		
			localTime = localtime(&long_time);
			//date is an ofs from 01 Jan 1900 00:00:00			
			sprintf(str,"%02d %s %02d %02d:%02d",
			localTime->tm_mday,str_month[localTime->tm_mon],localTime->tm_year-100,localTime->tm_hour,localTime->tm_min);
			if (i) 
			{
				menu_messageRaw(300,50+i*30,str);
				menu_button(160,50+i*30,"X",5,&item_rect[6+i],3);
			}
			else 
			{
				menu_messageRaw(300,170,str);
				menu_button(160,170,"X",5,&item_rect[6+i],3);
			}
	  	}		  		  	
  	  
  	  	  	  	  	  	
	  menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);
	  menu_button(10,80,(char*)"SAVE",0,&item_rect[1],2);
	  menu_button(200,80,(char*)"SLOT 1",(slot[1]?2:1),&item_rect[2],1);
	  menu_button(200,110,(char*)"SLOT 2",(slot[2]?2:1),&item_rect[3],1);
	  menu_button(200,140,(char*)"SLOT 3",(slot[3]?2:1),&item_rect[4],1);
	  menu_button(200,170,(char*)"QUICK",(slot[0]?2:1),&item_rect[5],1);
	  if (scr) WinScreenUnlock();
  	  }
	  menu_waitPen(&x,&y,1);
	  
	  if (TEST_PEN_MENU(x,y,item_rect[6])) 
	  {
	  	if (menu_confirm("Delete QUICKSLOT?"))
	  	{
		  	char str[255];		
			getstate_filename(0,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[7])) 
	  {
	  	if (menu_confirm("Delete SLOT 1?"))
	  	{
		  	char str[255];		
			getstate_filename(1,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[8])) 
	  {
	  	if (menu_confirm("Delete SLOT 2?"))
	  	{
		  	char str[255];		
			getstate_filename(2,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[9])) 
	  {
	  	if (menu_confirm("Delete SLOT 3?"))
	  	{
		  	char str[255];		
			getstate_filename(3,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}
	  //SAVE 0
	  if (TEST_PEN_MENU(x,y,item_rect[2]))
	  {
	  	  if (!slot[1]||(menu_confirm("Overwrite ?")))
		  {state_save(1);result=1;break;}
		  else redraw=1;
		  
	  }
	  //SAVE 1
	  if (TEST_PEN_MENU(x,y,item_rect[3]))
	  {
		  if (!slot[2]||(menu_confirm("Overwrite ?")))
		  {state_save(2);result=1;break;}
		  else redraw=1;
		
	  }
	  //SAVE 2
	  if (TEST_PEN_MENU(x,y,item_rect[4]))
	  {
    	  if (!slot[3]||(menu_confirm("Overwrite ?")))
		  {state_save(3);result=1;break;}
		  else redraw=1;
		
	  }
	  //QUICKSAVE
	  if (TEST_PEN_MENU(x,y,item_rect[5]))
	  {
	  	  if (!slot[0]||(menu_confirm("Overwrite ?")))
		  {state_save(0);result=1;break;}
		  else redraw=1;
		
	  }
  }  
  ljz_free(item_rect);
  return result;
}



int oswanZ_ShowMenuLoad(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
   int slot[4];
  RectangleType *item_rect;
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));
  memset((char*)item_rect,0,15*sizeof(RectangleType));
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr&&savescr) memcpy(scr,savescr,480*320*2);	    
//		if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  	
  	  FILE *savefile;
  	 
  	  slot[0]=slot[1]=slot[2]=slot[3]=0;
      for (int i=0;i<4;i++) 
      	if (!state_load(i,1))
      		{
      		slot[i]=1;
      
	  		struct tm *localTime;
			time_t long_time;
			char str[255];		
			getstate_filename(i,str);
			long_time=__getdate_file(str);		
			localTime = localtime(&long_time);
			//date is an ofs from 01 Jan 1900 00:00:00			
			sprintf(str,"%02d %s %02d %02d:%02d",
			localTime->tm_mday,str_month[localTime->tm_mon],localTime->tm_year-100,localTime->tm_hour,localTime->tm_min);
			if (i) 
			{
				menu_messageRaw(300,50+i*30,str);
				menu_button(160,50+i*30,"X",5,&item_rect[6+i],3);
			}
			else 
			{
				menu_messageRaw(300,170,str);
				menu_button(160,170,"X",5,&item_rect[6+i],3);
			}
	  	}	
      
	  menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);
	  menu_button(10,80,(char*)"LOAD",0,&item_rect[1],2);
	  menu_button(200,80,(char*)"SLOT 1",(slot[1]?2:3),&item_rect[2],1);
	  menu_button(200,110,(char*)"SLOT 2",(slot[2]?2:3),&item_rect[3],1);
	  menu_button(200,140,(char*)"SLOT 3",(slot[3]?2:3),&item_rect[4],1);
	  menu_button(200,170,(char*)"QUICK",(slot[0]?2:3),&item_rect[5],1);
		if (scr) WinScreenUnlock();	  
  	  }
	  menu_waitPen(&x,&y,1);
	  
	  
	  if (TEST_PEN_MENU(x,y,item_rect[6])) 
	  {
	  	if (menu_confirm("Delete QUICKSLOT?"))
	  	{
		  	char str[255];		
			getstate_filename(0,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[7])) 
	  {
	  	if (menu_confirm("Delete SLOT 1?"))
	  	{
		  	char str[255];		
			getstate_filename(1,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[8])) 
	  {
	  	if (menu_confirm("Delete SLOT 2?"))
	  	{
		  	char str[255];		
			getstate_filename(2,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  if (TEST_PEN_MENU(x,y,item_rect[9])) 
	  {
	  	if (menu_confirm("Delete SLOT 3?"))
	  	{
		  	char str[255];		
			getstate_filename(3,str);
	  		remove(str);
	  	}
	  	redraw=1;
	  }
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}
	  //LOAD 0
	  if (TEST_PEN_MENU(x,y,item_rect[2]))
	  {
		  if (slot[1])
		if (!state_load(1,0)) {result=1;break;}
	  }
	  //LOAD 1
	  if (TEST_PEN_MENU(x,y,item_rect[3]))
	  {
		  if (slot[2])
		if (!state_load(2,0)) {result=1;break;}
	  }
	  //LOAD 2
	  if (TEST_PEN_MENU(x,y,item_rect[4]))
	  {
		  if (slot[3])
		if (!state_load(3,0)) {result=1;break;}	  
	  }
	
	  //QUICKLOAD
	  if (TEST_PEN_MENU(x,y,item_rect[5]))
	  {
	  	if (slot[0])
    	if (!state_load(0,0)) {result=1;break;}
	  }
  }  
  ljz_free(item_rect);
  return result;
}



int oswanZ_ShowSysInfos(uint8 *savescr)
{
  uint8 *scr;
  int x,y;
  int redraw;  
  int result;
  RectangleType *item_rect,my_rect;
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));
	    
  redraw=1;         
  for (;;)
  {  
  	  if (redraw)
  	  {
  	  	redraw=0;
  	  	scr=WinScreenLock(winLockCopy);  
	    if (scr&&savescr) memcpy(scr,savescr,480*320*2);	    
		//if (scr) WinScreenUnlock();
  	  	RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
				
		char str[64],strb[32];
		unsigned long maxP,freeP,usedP;		
		VolumeInfoType vinfo;
		VFSVolumeSize(oswanZ_curVolume,&usedP,&maxP);		
		VFSVolumeGetLabel(oswanZ_curVolume,strb,16);strb[16]=0;
		menu_button(10,70,"Volume",0,&my_rect,4);		
		sprintf(str,"%s[%d] Used:%dMb/%dMb",strb,oswanZ_curVolume,usedP>>20,maxP>>20);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,70,str);				
		MemHeapFreeBytes(0,&freeP,&maxP);		
		menu_button(10,100,"RAM",6,&my_rect,4);		
		sprintf(str,"RAM Free:%d Max:%d",freeP,maxP);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,100,str);		
		TwGfxInfoType gfxinfo;
		gfxinfo.size=sizeof(gfxinfo);
		TwGfxGetInfo(oswanZ_mGfxLib,&gfxinfo);		
		menu_button(10,130,"VRAM",6,&my_rect,4);		
		sprintf(str,"Free:%d Max:%d",gfxinfo.freeAcceleratorMemory,gfxinfo.totalAcceleratorMemory);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,130,str);
						

		ws_romHeaderStruct 	*romHeader=ws_rom_getHeader(ws_rom,romSize);
		
		menu_button(10,160,"ROM",4,&my_rect,4);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,160,shortrom_filename);
		
		
		menu_button(10,190,"Size",4,&my_rect,4);
		sprintf(str,"%dMbits",(romSize>>20)<<3,strb);		
	    menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,190,str);
	    
		menu_button(200,190,"Min.Sys.",4,&my_rect,4);		
		sprintf(str,"%s",(romHeader->minimumSupportSystem==WS_SYSTEM_MONO)?"Wonderswan mono":"Wonderswan color");
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,190,str);
		
		menu_button(10,220,"Cart ID",7,&my_rect,4);		
		sprintf(str,"0x%.2x",romHeader->cartId);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,220,str);
		
		menu_button(200,220,"Developper ID",7,&my_rect,4);		
		sprintf(str,"0x%.2x",romHeader->developperId);
		menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,220,str);
		
		menu_button(10,250,"EEPROM",4,&my_rect,4);
		strb[0]=0;				
		switch (romHeader->eepromSize&0xf)
		{
			case WS_EEPROM_SIZE_NONE:	sprintf(strb,"none"); break;
			case WS_EEPROM_SIZE_64k:	sprintf(strb,"64 Kb"); break;
			case WS_EEPROM_SIZE_256k:	sprintf(strb,"256 Kb"); break;
		}
		sprintf(str,"%s",strb);		
	    menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,250,str);
		
		strb[0]=0;
		switch (romHeader->eepromSize&0xf0)
		{
			case WS_SRAM_SIZE_NONE:	sprintf(strb,"none"); break;
			case WS_SRAM_SIZE_1k:	sprintf(strb,"1 Kb"); break;
			case WS_SRAM_SIZE_8k:	sprintf(strb,"8 Kb"); break;
			case WS_SRAM_SIZE_16k:	sprintf(strb,"16 Kb"); break;
		}
		menu_button(200,250,"SRAM",4,&my_rect,4);
		sprintf(str,"%s",strb);		
	    menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,250,str);
	    
	    
	    menu_button(10,280,"RTC",4,&my_rect,4);
		sprintf(str,"%s",(romHeader->realtimeClock)?"Yes":"None");		
	    menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,280,str);
	    
	    menu_button(200,280,"Checksum",4,&my_rect,4);
		sprintf(str,"0x%.4x",romHeader->checksum);		
	    menu_messageRaw(my_rect.topLeft.x+my_rect.extent.x+5,280,str);


	ljz_free(romHeader);    		
		
						  	
	  	menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);
		
	  
	  	if (scr) WinScreenUnlock();
  	  }
	  menu_waitPen(&x,&y,1);
	  
	  
	  
	  //BACK	  
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}
	  
	  
  }  
  ljz_free(item_rect);
  return result;
}


int oswanZ_ShowMenu()
{
  int x,y;
  int redraw;
  int result;
  int slot0;
  uint8 *scr,*savescr;
  RectangleType *item_rect;
  item_rect=(RectangleType*)ljz_malloc(15*sizeof(RectangleType));  
  redraw=1;
  
  oswanZ_pause(1);
    
  
  TwGfxSurfaceType* displaySurface;
  // Get palmos display surface:
  xassert(TwGfxGetPalmDisplaySurface(oswanZ_mGfxLib, &displaySurface));
  while (TwGfxIsSurfaceReady(displaySurface));

  
  savescr=(uint8*)ljz_malloc(320*480*2);  
  scr=WinScreenLock(winLockCopy);  
  if (scr&&savescr)
  {
	  for (x=0;x<320*480;x++)
	  {
  		y=((uint16*)scr)[x];
	  	((uint16*)scr)[x]=((y>>12)<<11)|(((y>>6)&31)<<5)|((y>>1)&15);
	  }
	  memcpy(savescr,scr,480*320*2);
  }
  if (scr) WinScreenUnlock();
            
              
  for (;;)
  {  
  	if (redraw)
  	{  	
	  	slot0=!state_load(0,1);    
  		redraw=0;
		scr=WinScreenLock(winLockCopy);  
	    if (scr&&savescr) memcpy(scr,savescr,480*320*2);	    
//		if (scr) WinScreenUnlock();
  		
  		RGBColorType fg;
	  	WinPushDrawState ();
  	  	WinSetDrawMode(winOverlay);
    	fg.r=50;fg.g=200;fg.b=50;
	    WinSetTextColorRGB(&fg,NULL);
	    WinSetForeColorRGB(&fg,NULL);
	    FntSetFont(boldFont);
		WinPaintChars(LJZ_OSD_TITLE,strlen(LJZ_OSD_TITLE),10,10);	
		WinPopDrawState();
  		
		menu_button(310,20,(char*)"BACK",6,&item_rect[0],2);
		menu_button(60,100,(char*)"SAVE",0,&item_rect[1],2);
		menu_button(280,100,(char*)"QUICK SAVE",(slot0?2:1),&item_rect[2],2);
		menu_button(60,150,(char*)"LOAD",0,&item_rect[3],2);
		menu_button(280,150,(char*)"QUICK LOAD",(slot0?2:3),&item_rect[4],2);
		menu_button(60,200,(char*)"CONFIG",0,&item_rect[5],2);
		menu_button(280,250,(char*)"LAUNCHER",4,&item_rect[6],2);
		
		menu_button(280,200,(char*)"SYS INFOS",7,&item_rect[8],2);
		
		menu_button(60,250,(char*)"RESET WS",9,&item_rect[7],2);				
		if (scr) WinScreenUnlock();		
		
  	}
	  menu_waitPen(&x,&y,1);
	  //HOME
	  if (TEST_PEN_MENU(x,y,item_rect[6])) {result=1;break;}
	  //BACK
	  if (TEST_PEN_MENU(x,y,item_rect[0])) {result=0;break;}
	  //CONFIG
	  if (TEST_PEN_MENU(x,y,item_rect[5])) {oswanZ_ShowMenuConfig(savescr);redraw=1;}
	  //LOAD
	  if (TEST_PEN_MENU(x,y,item_rect[3])) {if (oswanZ_ShowMenuLoad(savescr)) {result=0;break;} redraw=1;}
	  //SAVE
	  if (TEST_PEN_MENU(x,y,item_rect[1])) {if (oswanZ_ShowMenuSave(savescr)) {result=0;break;} redraw=1;}
	  //QUICK SAVE
	  if (TEST_PEN_MENU(x,y,item_rect[2]))
	  {
		  state_save(0);
		  result=0;
		  break;
	  }
	  //QUICK LOAD
	  if (TEST_PEN_MENU(x,y,item_rect[4]))
	  {
		  if (slot0)
		    if (!state_load(0,0)) {result=0;break;}
	  }
	  //RESET WS
	  if (TEST_PEN_MENU(x,y,item_rect[7])) {if (menu_confirm("Reset WS ?")) {ws_reset(); result=0;break;}redraw=1;}
	  
	  	  //SYS INFOS
	  if (TEST_PEN_MENU(x,y,item_rect[8])) {oswanZ_ShowSysInfos(savescr);redraw=1;}

	}
  ljz_free(item_rect);
  
  ljz_free(savescr);
            
  oswanZ_pause(0);
  return result;
}  

void oswan_updateInput(void)
{

	ws_key_button_1=0;	
	if (oswanZ_padState&KEY_A)
   	 switch (oswanZ_autofireA)
	 {
  		case 0:ws_key_button_1=1; break;
  		case 1:
  	  		if ((oswanZ_framecount&31)>15) ws_key_button_1=1;
  	  		break;
  		case 2:
  	  		if ((oswanZ_framecount&15)>7) ws_key_button_1=1;
  	  		break;
  		case 3:
  	  		if ((oswanZ_framecount&7)>3) ws_key_button_1=1;
  	  		break;  		   
  	 }
  	 
  	ws_key_button_2=0;	
  	if (oswanZ_padState&KEY_B)
	 switch (oswanZ_autofireB)
	 {
  		case 0:ws_key_button_2=1; break;
  		case 1:
  	  		if ((oswanZ_framecount&31)>15) ws_key_button_2=1;
  	  		break;
  		case 2:
  	  		if ((oswanZ_framecount&15)>7) ws_key_button_2=1;
  	  		break;
  		case 3:
  	  		if ((oswanZ_framecount&7)>3) ws_key_button_2=1;
  	  		break;  		   
  	 }
	
	

	if (oswanZ_padState&KEY_START) ws_key_start=1;
	else ws_key_start=0;
	if (oswanZ_padState&KEY_UP) ws_key_up=1;
	else ws_key_up=0;
	if (oswanZ_padState&KEY_DOWN) ws_key_down=1;
	else ws_key_down=0;
	if (oswanZ_padState&KEY_LEFT) ws_key_left=1;
	else ws_key_left=0;
	if (oswanZ_padState&KEY_RIGHT) ws_key_right=1;
	else ws_key_right=0;
	if (oswanZ_padState&KEY_FLIPPED) ws_key_flipped=1;
	else ws_key_flipped=0;
	
	if (oswanZ_padState&KEY_TURBO)
	{		
		oswanZ_fastforward=1;		
	}
	else 
	if (oswanZ_fastforward)
	{
		oswanZ_fastforward=0;
		oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
	}
	
	if (oswanZ_padState&KEY_QUICKLOAD)
	{
		if (!state_load(0,1))
		{
			if (menu_confirm("Load state ?")) state_load(0,0);
			oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
		}
	}
	if (oswanZ_padState&KEY_QUICKSAVE)
	{
		if (menu_confirm("Save state ?")) state_save(0);
			oswanZ_frameBeginTime=oswanZ_getTickCount()+oswanZ_frameTime*100/oswanZ_speedpct;
	}
}



