/*
	File:			ljZ.cpp	

	Description:	ljZ demo source file. This is where all the magic happens.

	Author:		

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/
#include <TapWave.h>

#ifdef EOF
#undef EOF
#endif
#ifdef WIN32
#define snprintf _snprintf
#endif

#ifndef __MWERKS__
#include "time.h"
#endif

#include <stdio.h>
#include <string.h>

#include "math.h"


#include "ljz_std.h"


extern "C" int genZ_loadrom();
extern "C" void genZ_end();

extern "C" int genZ_setupenv(TwGfxType* mGfxLib);

extern "C" void genZ_changefskip(void);
extern "C" void genZ_changerendering(void);
extern "C" void genZ_updatevpad(unsigned int val);
extern "C" void genZ_MainLoop(void);
extern "C" void genZ_pause(int pause);
extern "C" int genZ_ShowMenu(void);

#define KEY_UP    		0x00000001
#define KEY_DOWN  		0x00000002
#define KEY_LEFT  		0x00000004
#define KEY_RIGHT 		0x00000008
#define KEY_SEL   		0x00000010
#define KEY_A     		0x00000020
#define KEY_B	  		0x00000040
#define KEY_C	 		0x00000080
#define KEY_START		0x00000100
#define KEY_QUICKLOAD   0x00000200
#define KEY_QUICKSAVE   0x00000400
#define KEY_TURBO  		0x00000800
#define KEY_SCREENMODE	0x00001000
#define KEY_FRAMESKIP	0x00002000


unsigned int g_translateBtn[21]=
{KEY_A,KEY_B,KEY_C,KEY_START,
KEY_UP,KEY_DOWN,KEY_LEFT,KEY_RIGHT,
KEY_QUICKLOAD,KEY_QUICKSAVE,KEY_FRAMESKIP,
KEY_SCREENMODE,KEY_TURBO,0,0,0,0};
unsigned char g_confinput[20];

// Assertion Support
static void
assert_check(int error, const char* message)
{
    if (error) {
        ErrDisplayFileLineMsg("PalmOS", (UInt16) error, message);
    }
}

#define xassert(error) {assert_check(error, #error);}

static void showError(const char* aWhat, Err aError) {
    ErrDisplayFileLineMsg("oink", (UInt16) aError, aWhat);
}

static void showError(const char* aWhat, UInt32 aValue) {
    char chbuf[50];
    StrPrintF(chbuf, "%lx", aValue);
    ErrDisplayFileLineMsg(aWhat, (UInt16)0, chbuf);
}

//----------------------------------------------------------------------

// lame lame lame random number generator
UInt32 gRandomSeed;
static Int32 random() 
{
    UInt32 result;

    // Generate the new seed
    gRandomSeed *= 2147001325;
    gRandomSeed += 715136305;

    // Return the random number
    result = gRandomSeed >> 16;

    return Int32(result);
}

//----------------------------------------------------------------------
// Simple base class that zeros memory

class ZeroBase {
public:
    // Override operator new and delete for this class so that operator new can zero out the
    // instance data.
    void* operator new(size_t aSize) {
        void* p = MemPtrNew(aSize);
        if (p) {
            MemSet(p, aSize, 0);
        }
        return p;
    }

    void operator delete(void* aPtr) {
        MemPtrFree(aPtr);
    }
};

//----------------------------------------------------------------------

// Simple sample application class that wraps up the various Tapwave Native Application calls
// to create a well formed application. In theory this could be implemented in a library, or a
// template.

static class GenericApp* gApp;

static SYSTEM_CALLBACK Boolean ClassAppHandleEvent(EventType* aEvent);

class GenericApp : public ZeroBase {
public:
    // Note: set the default delay between event loops to 10ms
    GenericApp(Int32 aDelay) : mDelay(aDelay) {
    }

    virtual ~GenericApp() {
    }

    virtual void Launch() {
        AppStart();
        AppEventLoop();
        AppStop();
    }

    virtual void AppStart() {
        TwAppStart(&ClassAppHandleEvent);
        AppBegin();
    }

    // Install your application specific initialization code in the AppBegin method
    virtual void AppBegin() {		
    }

    virtual void AppEventLoop() {
        TwAppRun(&mDelay);
    }

    virtual void AppStop() {
        AppEnd();
        TwAppStop();
    }

    // Install your application specific cleanup code in the AppEnd method
    virtual void AppEnd() {		
    }

    virtual Boolean AppHandleEvent(EventType* aEvent) {
        RectangleType bounds;
        Boolean handled = false;

        switch (aEvent->eType) {
          case winDisplayChangedEvent:
            // This logic responds to the user changing the orientation or the size of the
            // display area by hiding or showing the grafitti area.
            WinGetBounds(WinGetDisplayWindow(), &bounds);
            WinSetBounds(WinGetActiveWindow(), &bounds);
            HandleResizeEvent();
            handled = true;
            break;

          case keyDownEvent:
            // This logic responds to the user pressing the function button which we use to
            // make the application become fullscreen or revert to non-fullscreen (as a
            // toggle).
            if (aEvent->data.keyDown.chr == vchrFunction) {
                UInt32 visible;
                StatGetAttribute(statAttrBarVisible, &visible);
                if (visible) {
                    PINSetInputAreaState(pinInputAreaClosed);
                    StatHide();
                } else {
                    StatShow();
                    PINSetInputAreaState(pinInputAreaUser);
                }
                handled = true;
            }
            break;

          default:
            break;
        }
        return handled;
    }

    // This method is called when the display area changes size
    virtual void HandleResizeEvent() {
    }

protected:
    // Delay between event loops
    Int32 mDelay;
};

// Note: this cannot be a static class method because the SYSTEM_CALLBACK attribute doesn't
// work there.
static SYSTEM_CALLBACK Boolean ClassAppHandleEvent(EventType* aEvent) {
    return gApp->AppHandleEvent(aEvent);
}

//----------------------------------------------------------------------

    // What a lovely bunch of magic numbers:
    //  450 => display width
    //  344x84 => tapwave image dimensions
    //  44x34 => go, do image dimensions
    //  52x34 => do image dimensions
    //  64x34 => play image dimensions

#define DISPLAY_WIDTH 450
#define DISPLAY_HEIGHT 320

#define FRAMES_PER_SECOND       60
#define APP_MDELAY_NTSC			17
#define APP_MDELAY_PAL			20
#ifdef __MWERKS__
	#define APP_MDELAY			0
#else
	#define APP_MDELAY			8
#endif

//----------------------------------------------------------------------



//----------------------------------------------------------------------

class ljZ : public GenericApp {
public:
    ljZ() : GenericApp(APP_MDELAY) {
    }

    virtual ~ljZ() {
    }

	
	
    virtual void AppStart() {
		// Get the screen to a state we want.		        
//        WinSetCoordinateSystem(kCoordinatesNative);
		SysSetOrientation(sysOrientationLandscape);
        PINSetInputAreaState(pinInputAreaHide);
                

		StatHide();
		AppBegin();
    }

    typedef struct {
		Int32 mUp;
		Int32 mDown;
		Int32 mLeft;
		Int32 mRight;
		Int32 mSelect;
        Int32 mActionA;
        Int32 mActionB;
		Int32 mActionC;
        Int32 mActionD;
		Int32 mTriggerL;
        Int32 mTriggerR;
		Int32 mFunc;        
		Int32 mLaunch;
		Int32 mPenX;
		Int32 mPenY;
		Int32 mPenZ;
		Int32 mPower;
    } UserEvent;

    virtual void AppBegin() {
        Int32 inputFormats[] = { twInputNavUp,twInputNavDown,twInputNavLeft,twInputNavRight,twInputNavSelect,twInputActionA, twInputActionB,twInputActionC,twInputActionD,twInputTriggerA,twInputTriggerB,twInputFunction,twInputLaunch,twInputPenX,twInputPenY,twInputPenZ,twInputPower };
        
#ifdef __DEBUG__
	UInt32 flags = AdnDebugEnableGet();
	flags |= kAdnEnableMasterSwitch | kAdnEnableFullDebugging;
	AdnDebugEnableSet(flags);
		
	// Set memory location for debugger
	AdnDebugNativeRegister(sysFileTApplication, 'LJZZ', 'ARMC', 3);
//
#endif
        
        

        xassert(TwGfxOpen(&mGfxLib, NULL));
        ClearScreen();
        genZ_setupenv(mGfxLib);


// List of key bindings

#define keysAllowedMask (keyBitsAll)

        // Set the keys we poll to not generate events.  This saves cpu cycles
        // by avoiding the events flowing through the event loop which we will 
        // ignore anyways.
        KeySetMask(~keysAllowedMask);

        TwInputOpen(&mInputQueue, "twinput", "r");

        mInputFormat = new Int32[sizeof(inputFormats)/sizeof(inputFormats[0])];
        MemMove(mInputFormat, inputFormats, sizeof(inputFormats));
        TwInputSetFormat(mInputQueue, mInputFormat, sizeof(inputFormats));
        mLastPenZ=0;
            
        // Activating the queue stops the joystick from generating
        // keyDownEvents.  If this is not done, then every direction
        // visited while polling will cause a corresponding keyDownEvent
        // in that direction, and if the polling duration exceeds 
        // the keyDown repeat duration, then a repeating keyDownEvent 
        // will be generated too.
        TwInputActivate(mInputQueue);		
		mStopEmulation=genZ_loadrom();		
		ClearScreen();
		
		unsigned char *ftrptr;
		FtrGet(APPLI_ID,FTR_OPT_INPUTCONF_ID ,(unsigned long*)&ftrptr);
		for (int i=0;i<20;i++) g_confinput[i]=ftrptr[i];
    }


    virtual void AppStop()
    {
    	AppEnd();
    }


    virtual void AppEnd() {        		    			
        TwInputClose(mInputQueue);
        delete mInputFormat;
		genZ_end();		
		
		StatShow();		
		//PINSetInputAreaState(pinInputAreaShow);

		
			
        TwGfxClose(mGfxLib);
        
        KeySetMask(keysAllowedMask);
    }

    virtual void AppEventLoop() {
	    int keyval;
	    int pollcpt=0;
        for (;;)
		{			
			pollcpt++;
			//if (!(pollcpt&1))
			{
				keyval=0;
				if (!TwInputPoll(mInputQueue, (Int32 *) &mUserEvent, sizeof(mUserEvent)))
				{			
				  
				  if (mUserEvent.mUp) keyval|=g_translateBtn[g_confinput[BTN_Up]];// KEY_UP;
				  if (mUserEvent.mDown) keyval|=g_translateBtn[g_confinput[BTN_Down]];//KEY_DOWN;
				  if (mUserEvent.mLeft) keyval|=g_translateBtn[g_confinput[BTN_Left]];//KEY_LEFT;
				  if (mUserEvent.mRight) keyval|=g_translateBtn[g_confinput[BTN_Right]];//KEY_RIGHT;
				  if (mUserEvent.mActionA) keyval|=g_translateBtn[g_confinput[BTN_Blue]];//KEY_A;
				  if (mUserEvent.mActionB) keyval|=g_translateBtn[g_confinput[BTN_Yellow]];//KEY_B;
				  if (mUserEvent.mActionC) keyval|=g_translateBtn[g_confinput[BTN_Green]];//;
				  if (mUserEvent.mActionD) keyval|=g_translateBtn[g_confinput[BTN_Red]];//KEY_D;
				  if (mUserEvent.mLaunch) keyval|=g_translateBtn[g_confinput[BTN_Home]];//KEY_SEL;
				  if (mUserEvent.mFunc) keyval|=g_translateBtn[g_confinput[BTN_Func]];//KEY_FUNC;
				  if (mUserEvent.mSelect) keyval|=g_translateBtn[g_confinput[BTN_Anal]];//KEY_ANAL;
				  if (mUserEvent.mTriggerL) keyval|=g_translateBtn[g_confinput[BTN_L]];//KEY_L;
				  if (mUserEvent.mTriggerR) keyval|=g_translateBtn[g_confinput[BTN_R]];//KEY_R;
				  
				  if (mUserEvent.mPower) 
				  {
				  	  genZ_pause(1);
					  EventType myevent;
						for (;;)
						{
							EvtGetEvent(&myevent,0);
							if (myevent.eType==nilEvent) break;
							if (myevent.eType==keyDownEvent)
							{
								if (myevent.data.keyDown.chr!=vchrLaunch) SysHandleEvent(&myevent);
							}
						}
					   genZ_pause(0);
				  	
				  }
				  
				  
				  if ((mUserEvent.mPenZ)/*&&(!mLastPenZ)*/)
				  {				  	 					 
				 	if ((mUserEvent.mPenY<60)&&(mUserEvent.mPenX>400))
					{
						keyval|=g_translateBtn[g_confinput[BTN_TR]];
					}
					else
					if ((mUserEvent.mPenY>260)&&(mUserEvent.mPenX>400))
					{
						keyval|=g_translateBtn[g_confinput[BTN_BR]];						
					}
					else
					if ((mUserEvent.mPenY<60)&&(mUserEvent.mPenX<80))
					{
						keyval|=g_translateBtn[g_confinput[BTN_TL]];
					}
					else
					if ((mUserEvent.mPenY>260)&&(mUserEvent.mPenX<80))
					{
						keyval|=g_translateBtn[g_confinput[BTN_BL]];
					}
					else
					{
						switch (genZ_ShowMenu())
						{
							case 1:mStopEmulation=1;break;
						}
						ClearScreen();
					}															
				  }
				  mLastPenZ=mUserEvent.mPenZ;				  				  
				  genZ_updatevpad(keyval);
				  if (keyval&KEY_FRAMESKIP) 
		  		  {
					genZ_changefskip();
					ClearScreen();
				  }
				  if (keyval&KEY_SCREENMODE) 
				  {
					genZ_changerendering();
					ClearScreen();
				  }
				}
			}
			LJZ_HANDLE_SYS_EVENT(!(pollcpt&31),genZ_pause)
			if (!mStopEmulation) genZ_MainLoop();
			else break;
		}
    }

    virtual void HandleResizeEvent() {

		SysSetOrientation(sysOrientationLandscape);
        PINSetInputAreaState(pinInputAreaHide);				        
        ClearScreen();
    }
    
    void ClearScreen() {
		//clear
		ljz_cls(mGfxLib,TwGfxComponentsToPackedRGB(0,0,0));	
    }
    

protected:
    TwGfxType* mGfxLib;        
		
    TwInputHandle mInputQueue;
    Int32* mInputFormat;    
	UserEvent mUserEvent;	
	Int32 mLastPenZ;
	Int32 mStopEmulation;
};

//----------------------------------------------------------------------

// Palm entry point

extern "C" UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    if (cmd == sysAppLaunchCmdNormalLaunch) {

        gApp = new ljZ();
        gApp->Launch();
        delete gApp;
    }

    return 0;
}
